﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;

namespace KellermanSoftware.Common
{
    /// <summary>
    /// Create a CSV string or output to a writer.
    /// Supports lists of objects and data tables
    /// Supports default headers and special character escaping
    /// Supports reading CSV Strings, Files, and Streams
    /// </summary>
    public class CsvHelper
    {
        #region Constructor
        public CsvHelper()
        {
            OutputHeader = true;
            InsertHeaderSpaces = true;
            Seperator = ",";
        }
        #endregion

        #region Properties
        /// <summary>
        /// If true, a default header will be output. Default is true
        /// </summary>
        public bool OutputHeader { get; set; }

        /// <summary>
        /// If true, Pascal Cased Header Names will be split into words. Default is true
        /// </summary>
        public bool InsertHeaderSpaces { get; set; }

        /// <summary>
        /// The seperator for the columns.  Default is a comma
        /// </summary>
        public string Seperator { get; set; }
        #endregion

        #region Methods

        /// <summary>
        /// Read CSV String with a header
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public List<Dictionary<string, string>> ReadCsvString(string value)
        {
            MemoryStream memoryStream = new MemoryStream();
            TextWriter textWriter = new StreamWriter(memoryStream);
            textWriter.Write(value);
            textWriter.Flush();
            memoryStream.Seek(0, SeekOrigin.Begin);
            return ReadCsvStream(memoryStream);
        }

        /// <summary>
        /// Read CSV File with a header
        /// </summary>
        /// <param name="filePath"></param>
        /// <returns></returns>
        public List<Dictionary<string, string>> ReadCsvFile(string filePath)
        {
            using (FileStream fileStream = new FileStream(filePath, FileMode.Open, FileAccess.Read))
            {
                return ReadCsvStream(fileStream);
            }
        }

        /// <summary>
        /// Read CSF Stream with a header
        /// </summary>
        /// <param name="stream"></param>
        /// <returns></returns>
        public List<Dictionary<string, string>> ReadCsvStream(Stream stream)
        {
            //This regex will handle normal commas, plus commas inside double quotes, 
            //and pairs of double quotes inside double quotes.
            //Because this function reads line by line, it will not handle CRLF in double quotes
            Regex regex = new Regex("(?<=^|,)(\"(?:[^\"]|\"\")*\"|[^,]*)",RegexOptions.Compiled);

            List<Dictionary<string, string>> results = new List<Dictionary<string, string>>();

            using (StreamReader reader = new StreamReader(stream))
            {
                string headerLine = reader.ReadLine();
                if (string.IsNullOrEmpty(headerLine))
                    return new List<Dictionary<string, string>>();

                MatchCollection headerMatches= regex.Matches(headerLine);

                string line;

                while ((line = reader.ReadLine()) != null)
                {
                    MatchCollection wordMatches = regex.Matches(line);
                    Dictionary<string, string> row = new Dictionary<string, string>();
                    
                    for (int i = 0; i < Math.Min(headerMatches.Count, wordMatches.Count); i++)
                    {
                        string cleanWord = wordMatches[i].Value.Replace("\"\"", "\"")
                            .TrimStart(new[] {'"'}).TrimEnd(new[] {'"'});

                        row.Add(headerMatches[i].Value, cleanWord);                        
                    }

                    results.Add(row);
                }
            }

            return results;
        }

        /// <summary>
        /// Write the properties of a class list into a TextWriter Stream
        /// </summary>
        /// <typeparam name="T">Class type</typeparam>
        /// <param name="stream">The output text writer</param>
        /// <param name="list">List of class objects</param>
        /// <param name="propertyNameOrder">The order of the properties to output</param>
        public void ClassListToCsv<T>(TextWriter stream, IEnumerable<T> list, List<string> propertyNameOrder) where T : class
        {
            Type type = typeof(T);
            PropertyInfo[] properties = type.GetProperties();

            if (OutputHeader)
                stream.WriteLine(CreateHeader(propertyNameOrder));

            foreach (var item in list)
                stream.WriteLine(PropertiesToCsv(propertyNameOrder, properties, item));
        }

        /// <summary>
        /// Write the properties of a class list to a CSV string
        /// </summary>
        /// <typeparam name="T">Class type</typeparam>
        /// <param name="list">List of class objects</param>
        /// <returns>CSV String</returns>
        public string ClassListToCsv<T>(IEnumerable<T> list) where T : class
        {
            Type type = typeof(T);
            PropertyInfo[] properties = type.GetProperties();
            List<string> propertyNames = properties.Select(propertyInfo => propertyInfo.Name).ToList();
            return ClassListToCsv(list, propertyNames);
        }


        /// <summary>
        /// Write the properties of a class list to a CSV string
        /// </summary>
        /// <typeparam name="T">Class type</typeparam>
        /// <param name="list">List of class objects</param>
        /// <param name="propertyNameOrder">The order of the properties to output</param>
        /// <returns>CSV String</returns>
        public string ClassListToCsv<T>(IEnumerable<T> list, List<string> propertyNameOrder) where T : class
        {
            StringBuilder sb = new StringBuilder();
            Type type = typeof (T);
            PropertyInfo[] properties = type.GetProperties();

            if (OutputHeader)
                sb.AppendLine(CreateHeader(propertyNameOrder));

            foreach (var item in list)
                sb.AppendLine(PropertiesToCsv(propertyNameOrder, properties, item));

            return sb.ToString();
        }

        /// <summary>
        /// Write the rows of a data table to a CSV string
        /// </summary>
        /// <param name="table">The table to output</param>
        /// <returns></returns>
        public string DataTableToCsv(DataTable table)
        {
            return DataTableToCsv(table, GetColumnNamesFromDataTable(table));
        }

        /// <summary>
        /// Write the rows of a data table to a CSV string
        /// </summary>
        /// <param name="table">The table to outpout</param>
        /// <param name="columnNameOrder">The column names in order to output</param>
        /// <returns></returns>
        public string DataTableToCsv(DataTable table, List<string> columnNameOrder)
        {
            var sb = new StringBuilder();

            if (OutputHeader)
                sb.AppendLine(CreateHeader(columnNameOrder));

            foreach (DataRow row in table.Rows)
                sb.AppendLine(RowToCsv(row, columnNameOrder));

            return sb.ToString();
        }

        /// <summary>
        /// Output the rows of a data table to a text writer
        /// </summary>
        /// <param name="stream">The output text writer</param>
        /// <param name="table">The table to output</param>
        public void DataTableToCsv(TextWriter stream, DataTable table)
        {
            DataTableToCsv(stream, table, GetColumnNamesFromDataTable(table));
        }

        /// <summary>
        /// Output the rows of a data table to a text writer
        /// </summary>
        /// <param name="stream">The output text writer</param>
        /// <param name="table">The table to output</param>
        /// <param name="columnNameOrder">The order of the columns to output</param>
        public void DataTableToCsv(TextWriter stream, DataTable table, List<string> columnNameOrder)
        {
            if (OutputHeader)
                stream.WriteLine(CreateHeader(columnNameOrder));

            foreach (DataRow row in table.Rows)
                stream.WriteLine(RowToCsv(row, columnNameOrder));
        }

        /// <summary>
        /// Get the column names from a DataTable
        /// </summary>
        /// <param name="table"></param>
        /// <returns></returns>
        private List<string> GetColumnNamesFromDataTable(DataTable table)
        {
            List<string> columnNames = new List<string>();

            for (int i = 0; i < table.Columns.Count; i++)
            {
                columnNames.Add(table.Columns[i].ColumnName);
            }
            return columnNames;
        }



        /// <summary>
        /// Create a header based on the passed in column names
        /// </summary>
        /// <param name="columnNames">The names of the columns for the header</param>
        /// <returns></returns>
        private string CreateHeader(List<string> columnNames)
        {
            //if (InsertHeaderSpaces)
            //    return String.Join(Seperator, columnNames.Select(StringHelper.InsertSpaces).ToArray());

            return String.Join(Seperator, columnNames.ToArray());
        }

        /// <summary>
        /// Output a DataRow to a comma delimited string in the specified column order
        /// </summary>
        /// <param name="row"></param>
        /// <param name="columnNameOrder"></param>
        /// <returns></returns>
        private string RowToCsv(DataRow row, IEnumerable<string> columnNameOrder)
        {
            var sb = new StringBuilder();
            bool firstColumn = true;

            foreach (var columnName in columnNameOrder)
            {
                if (!firstColumn)
                    sb.Append(Seperator);

                int columnIndex = row.Table.Columns.IndexOf(columnName);
                sb.Append(EscapeString(row[columnIndex]));
                firstColumn = false;
            }

            return sb.ToString();
        }

        /// <summary>
        /// Output the properties of an object to a comma delimited string
        /// </summary>
        /// <param name="propertyNameOrder"></param>
        /// <param name="properties"></param>
        /// <param name="objectValue"></param>
        /// <returns></returns>
        private string PropertiesToCsv(IEnumerable<string> propertyNameOrder, PropertyInfo[] properties, object objectValue)
        {
            StringBuilder sb = new StringBuilder();
            bool firstColumn = true;

            foreach (var propertyName in propertyNameOrder)
            {
                PropertyInfo propertyInfo = properties.FirstOrDefault(o => o.Name == propertyName);

                if (propertyInfo == null)
                    throw new ArgumentException("PropertyName not found " + propertyName);

                if (!firstColumn)
                    sb.Append(Seperator);

                sb.Append(EscapeString(propertyInfo.GetValue(objectValue, null)));
                firstColumn = false;
            }

            return sb.ToString();
        }        

        /// <summary>
        /// Escape special characters
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        private string EscapeString(object value)
        {
            if (value == null)
                return string.Empty;

            string data = value.ToString();

            // CSV rules: http://en.wikipedia.org/wiki/Comma-separated_values#Basic_rules
            // From the rules:
            // 1. if the data has quote, escape the quote in the data
            // 2. if the data contains the delimiter (in our case ','), double-quote it
            // 3. if the data contains the new-line, double-quote it.

            if (data.Contains("\""))
            {
                data = data.Replace("\"", "\"\"");
            }

            if (data.Contains(","))
            {
                data = String.Format("\"{0}\"", data);
            }
            else if (data.Contains(Environment.NewLine))
            {
                data = String.Format("\"{0}\"", data);
            }

            return data;
        }
        #endregion
    }
}
