#region Includes
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Windows.Forms;
using KellermanSoftware.NetEmailValidation;
using KellermanSoftware.Common;
using Microsoft.Win32;

#endregion

namespace NetEmailValidationDemo
{
    public partial class frmMain : Form
    {
        #region Class Variables
        private EmailValidation _valid;
        private DateTime _start = DateTime.MinValue;
        private StringBuilder _sbGood;
        private StringBuilder _sbBad;
        private StringBuilder _sbLog;
        List<ValidationOptions> _options;
        #endregion

        #region Constructor
        public frmMain()
        {
            InitializeComponent();
        }
        #endregion

        #region Events

        /// <summary>
        /// Occurs when email validation is complete
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void CompletionEvent(object sender, AsyncCompletedEventArgs e)
        {
            //Invoke is required for a Winform App
            if (InvokeRequired)
            {                
                this.Invoke(new AsyncCompletedEventHandler(CompletionEvent), new object[] { sender, e });
                return;
            }

            //If we are cancelled update the status and the progress bar
            if (_valid.Cancel)
            {
                lblStatus.Text = "Cancelled";
                bar.Value = 0;
            }
            else //Update the status to complete
            {
                TimeSpan ts = DateTime.Now - _start;

                lblStatus.Text = string.Format("Finished. Validated {0} emails in {1} seconds", bar.Maximum, ts.TotalSeconds);
                bar.Value = bar.Maximum;
            }

            //Remove the event connections
            _valid.ProgressEvent -= new EmailValidation.ProgressChangedEventHandler(ProgressEvent);
            _valid.ValidationCompleteEvent -= new AsyncCompletedEventHandler(CompletionEvent);
            _valid.ResponseReceivedEvent -= new EmailValidation.ResponseReceivedEventHandler(ResponseReceivedEvent);

            //Set button states
            btnProcess.Enabled = true;
            btnCancel.Enabled = false;
        }

        /// <summary>
        /// Occurs as each email is processed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        public void ProgressEvent(object sender, ProgressEventArgs args)
        {
            //Required for a Winform app
            if (InvokeRequired)
            {
                this.Invoke(new EmailValidation.ProgressChangedEventHandler(ProgressEvent), new object[] { sender, args });
                return;
            }

            //Update progress depending upon what options are chosen
            //We update less often if it is going faster
            if (args.TotalEmailsToProcess <= 50
                || args.EmailsProcessed % 50 == 0
                || (_options.Contains(ValidationOptions.MailServerExists) 
                    && args.EmailsProcessed % 25 == 0)
                || _options.Contains(ValidationOptions.SmtpConnect))
            {
                //Update text boxes
                txtGoodEmails.Text = _sbGood.ToString();
                txtBadEmails.Text = _sbBad.ToString();
                txtLog.Text = _sbLog.ToString();
                ScrollToEnd();

                //Update progress bar
                bar.Maximum = args.TotalEmailsToProcess;
                bar.Value = args.EmailsProcessed;
                lblStatus.Text = args.ProgressPercentage.ToString() + "%";
                Application.DoEvents();
            }
        }


        /// <summary>
        /// An email was validated and a response received
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="result"></param>
        public void ResponseReceivedEvent(object sender, Result result)
        {
            if (InvokeRequired)
            {
                this.Invoke(new EmailValidation.ResponseReceivedEventHandler(ResponseReceivedEvent), new object[] { sender, result });
                return;
            }

            //Add to logging
            string emailLine = string.Format("{0}{1}", result.EmailAddress, Environment.NewLine);

            if (result.IsValid)
            {
                _sbGood.Append(emailLine);
            }
            else
            {
                _sbBad.Append(emailLine);
            }

            _sbLog.Append(result.Log);
        }

        /// <summary>
        /// User clickeed process button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnProcess_Click(object sender, EventArgs e)
        {
            try
            {
                if (!ValidateSettings())
                {
                    return;
                }

                if (txtEmailList.Text.Trim().Length == 0)
                {
                    MessageBox.Show("Please enter some email addresses","No email addresses",MessageBoxButtons.OK,MessageBoxIcon.Information);
                    return;
                }               

                //Set defaults
                _valid.FromEmail = txtFromEmail.Text;
                _valid.FromMailServer = txtFromEmailServer.Text;

                //Set Progress Items
                _start = DateTime.Now;
                _valid.Cancel = false;

                //Clear all text fields
                ClearLogs();

                //Disable the process and enable the cancel buttons
                btnProcess.Enabled = false;
                btnCancel.Enabled = true;

                txtLog.Text = "Checking connections..." + Environment.NewLine;
                Application.DoEvents();

                //Set default options                
                _options = _valid.BestConnectionOptions();
                if (_valid.BestConnectionMessage.Length > 0)
                {
                    MessageBox.Show(_valid.BestConnectionMessage, "Some options disabled", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }


                Application.DoEvents();
                txtLog.Text += "Extracting List..." + Environment.NewLine;

                //Attach Events                
                _valid.ProgressEvent += new EmailValidation.ProgressChangedEventHandler(ProgressEvent);
                _valid.ValidationCompleteEvent += new AsyncCompletedEventHandler(CompletionEvent);
                _valid.ResponseReceivedEvent += new EmailValidation.ResponseReceivedEventHandler(ResponseReceivedEvent);

                //Extract the list from the email list text box
                List<string> emailList = new List<string>();
                emailList.AddRange(txtEmailList.Text.Split(new char[] { '\r' }));

                txtLog.Text += "Spinning up Threads..." + System.Environment.NewLine;
                Application.DoEvents();

                //Call the asynchronous validation
                _valid.ValidateListAsync(emailList, _options);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Self Test Failed: " + ex.Message, "Self Test Failed", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                btnProcess.Enabled = true;
            }
        }

        /// <summary>
        /// User clicked cancel
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            _valid.Cancel = true;
        }

        private void btnPasteEmail_Click(object sender, EventArgs e)
        {
            txtEmailList.Text = Clipboard.GetText();
        }

        private void btnCopyEmail_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(txtEmailList.Text);
        }

        private void btnPasteGood_Click(object sender, EventArgs e)
        {
            txtGoodEmails.Text = Clipboard.GetText();
        }

        private void btnCopyGood_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(txtGoodEmails.Text);
        }

        private void btnPasteBad_Click(object sender, EventArgs e)
        {
            txtBadEmails.Text = Clipboard.GetText();
        }

        private void btnCopyBad_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(txtBadEmails.Text);
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            try
            {
                RegistryKey regKey = Registry.CurrentUser.CreateSubKey("Software\\KellermanSoftware\\EmailValidation");

                string userName = regKey.GetValue("UserName", string.Empty).ToString();
                string licenseKey = regKey.GetValue("License", string.Empty).ToString();

                //Load keys from the registry
                _valid = new EmailValidation(userName,licenseKey);

                //Example of licensed mode
                //_valid = new EmailValidation("place user name here from your receipt", "place license key here from your receipt"); 
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void registerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                frmRegister register = new frmRegister();

                register.ShowDialog();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void importToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                ImportEmailsFromCSV();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Import emails from CSV file and display in email list control
        /// </summary>
        private void ImportEmailsFromCSV()
        {
            string emails = string.Empty;

            openFileDialog1.Filter = "CSV Files (.csv)|*.csv";

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                txtEmailList.Text = string.Empty;
                CsvHelper csvHelper = new CsvHelper();

                List<Dictionary<string, string>> list = csvHelper.ReadCsvFile(openFileDialog1.FileName);

                foreach (Dictionary<string, string> dictionary in list)
                {
                    string value = string.Empty;
                    dictionary.TryGetValue("Email", out value);

                    if (!string.IsNullOrEmpty(value))
                        emails += value + Environment.NewLine;
                }

                txtEmailList.Text = emails;
            }
        }

        /// <summary>
        /// Scroll the text boxes to the end
        /// </summary>
        private void ScrollToEnd()
        {
            if (txtGoodEmails.Text.Length > 0)
            {
                txtGoodEmails.Select(txtGoodEmails.Text.Length - 1, 0);
                txtGoodEmails.ScrollToCaret();
            }

            if (txtBadEmails.Text.Length > 0)
            {
                txtBadEmails.Select(txtBadEmails.Text.Length - 1, 0);
                txtBadEmails.ScrollToCaret();
            }

            if (txtLog.Text.Length > 0)
            {
                txtLog.Select(txtLog.Text.Length - 1, 0);
                txtLog.ScrollToCaret();
            }
        }

        /// <summary>
        /// Clear the text boxes and the logs
        /// </summary>
        private void ClearLogs()
        {
            txtLog.Text = string.Empty;
            txtGoodEmails.Text = string.Empty;
            txtBadEmails.Text = string.Empty;
            bar.Maximum = 100;
            bar.Value = 0;
            lblStatus.Text = "0%";
            _sbGood = new StringBuilder(txtEmailList.Text.Length);
            _sbBad = new StringBuilder(txtEmailList.Text.Length);
            _sbLog = new StringBuilder(txtEmailList.Text.Length * 10);
            Application.DoEvents();
        }

        /// <summary>
        /// Check entered settings are correct
        /// </summary>
        private bool ValidateSettings()
        {
            bool result = true;

            errorProvider.SetError(txtFromEmailServer, string.Empty);
            errorProvider.SetError(txtFromEmail, string.Empty);

            errorProvider.Clear();

            if (string.IsNullOrEmpty(txtFromEmailServer.Text))
            {
                errorProvider.SetError(txtFromEmailServer, "Please enter a value");
                result = false;
            }

            if (string.IsNullOrEmpty(txtFromEmail.Text))
            {
                errorProvider.SetError(txtFromEmail, "Please enter a value");
                result = false;
            }

            return result;
        }

        #endregion

    }
}